;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                           ;
;   SYSTEM TREE BROWSER                     ;
;                                           ;
;   Compile with FASM for MenuetOS          ;
;                                           ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
   
   
use32
   
               org    0x0
               db     'MENUET01'              ; 8 byte id
               dd     0x01                    ; header version
               dd     START                   ; program start
               dd     I_END                   ; program image size
               dd     0x100000                ; required amount of memory
               dd     0x7f000                 ; stack
               dd     0x0,0x0                 ; param,icon
   
START:                          ; start of execution
   
    call read_directory
    call draw_window            ; at first, draw the window
   
still:
   
    mov  eax,10                 ; wait here for event
    int  0x40
   
    cmp  eax,1                  ; redraw request ?
    jz   red
    cmp  eax,2                  ; key in buffer ?
    jz   key
    cmp  eax,3                  ; button in buffer ?
    jz   button
   
    jmp  still
   
  red:                          ; redraw
    call draw_window
    jmp  still
   
  key:                          ; key
    mov  eax,2
    int  0x40
    cmp  ah,184
    je   pgup
    cmp  ah,183
    je   pgdn
    cmp  ah,178
    je   curup
    cmp  ah,177
    je   curdn
    cmp  ah,52
    je   view1
    cmp  ah,53
    je   view1
    cmp  ah,57
    je   delete
    cmp  ah,182
    je   delete
    cmp  ah,13
    jne  still
    mov  ah,[butid]
    jmp  noreadhd3
  curup:
    cmp  [butid],21
    jbe  curup1
    dec  [butid]
    inc  [drawid]
    call drawbuttons
    dec  [drawid]
    jmp  still
  curup1:
    mov  [butid],36
    jmp  pgup
  curdn:
    cmp  [butid],36
    jae  curdn1
    inc  [butid]
    inc  [drawid]
    call drawbuttons
    dec  [drawid]
    jmp  still
  curdn1:
    mov  [butid],21
    jmp  pgdn
   
  button:                       ; button
    mov  eax,17
    int  0x40
   
    cmp  ah,1                   ; button id=1 ?
    jnz  noclose
    mov  eax,0xffffffff         ; close this program
    int  0x40
  noclose:
    cmp  ah,21
    jb   noclose1
    cmp  ah,36
    ja   noclose1
    cmp  ah,[butid]
    je   dblclick
    mov  [butid],ah
    inc  [drawid]
    call drawbuttons
    dec  [drawid]
  dblclick:
    mov  eax,5
    mov  ebx,30
    int  0x40
    mov  eax,17
    int  0x40
    cmp  al,1
    je   still
    cmp  ah,[butid]
    jne  still
  noclose1:
    pusha                       ; clear info text area
    mov  eax,13
    mov  ebx,70*65536+300
    mov  ecx,265*65536+8
    mov  edx,0xffffff
    int  0x40
    popa
   
     cmp  ah,41                  ; button id=12 -> directory
     jb   noreadhd
   
    shr  eax,8
    sub  eax,41
    imul eax,12
   
    cmp  eax,[loclen]
    jge  still
   
    mov  [loclen],eax
     mov  [readblock],1
     call read_directory
    mov  [butid],21
    call drawbuttons
   
    jmp  still
   
  noreadhd:
   
    cmp  ah,13                  ; button id=13 ?
    jnz  noreadhd2
  pgup:
    mov  eax,[readblock]
    cmp  eax,1
    jz   nozer1
    dec  eax
    mov  [readblock],eax
    call read_directory
    inc  [drawid]
    call drawbuttons
    dec  [drawid]
   nozer1:
    jmp  still
   
  noreadhd2:
   
    cmp  ah,14                  ; button id=14 ?
     jnz  reread
   pgdn:
    mov  eax,[readblock]
    inc  eax
    mov  [readblock],eax
    call read_directory
    inc  [drawid]
    call drawbuttons
    dec  [drawid]
  nozer2:
    jmp  still
   
  reread:
    cmp  ah,2
    jne  view
    call read_directory
    call drawbuttons
    jmp  still
  view:
    cmp  ah,4
    je   delete
    cmp  ah,3
    jne  noreadhd3
  view1:
    mov  ah,[butid]
    jmp  yeseditor1
   
  noreadhd3:
    push eax
    mov  eax,5
    mov  ebx,20
    int  0x40
    pop  eax
    cmp  ah,21
    jge  yesnewdir
    jmp  nonewdir
  yesnewdir:
   
    pusha
    mov  al,ah
    and  eax,255
    sub  eax,21
    xor  edx,edx
    mov  ebx,62
    mul  ebx
    mov  esi,eax
    add  esi,fileinfo+7
   
    cmp  [esi],word 'OL'
    jz   folok
   
    cmp  [esi+14],word 'XT'          ; show txt, asm and inc files
    je   yeseditor
    cmp  [esi+14],word 'SM'
    je   yeseditor
    cmp  [esi+14],word 'NC'
    je   yeseditor
    jmp  noeditor
  yeseditor:
    popa
  yeseditor1:
    call getparam
   
    mov  eax,19
    mov  ebx,editor
    mov  ecx,param
    int  0x40
   
    jmp  still
   
  noeditor:
                                ; start application
   
    popa
    pusha
   
    call getparam
    mov  ebx,param
   
    cmp  [location+1],dword 'RAMD'
    jne  no_ramdisk_start
    mov  eax,19
    mov  ecx,0
    int  0x40
  no_ramdisk_start:
   
    cmp  [location+1],dword 'HARD'
    jne  no_harddisk_start
   
    mov  esi,param
    mov  edi,location+1
    add  edi,[loclen]
    mov  ecx,11
    cld
    rep  movsb
   
    mov  eax,31
    mov  ebx,location+24
    mov  ecx,[loclen]
    sub  ecx,12
    mov  edx,0x10000
    int  0x40
   
  no_harddisk_start:
   
    popa
    jmp  still
   
   
  folok:
   
    popa
   
    mov  al,ah
    and  eax,255
    sub  eax,21
    xor  edx,edx
    mov  ebx,32
    mul  ebx
    mov  esi,eax
    mov  edi,[loclen]
    inc  edi
    add  esi,data_area+1024
    cmp  [esi],word '..'     ; if '..'
    jnz  chdir1
    mov  eax,[loclen]
    sub  eax,12
    mov  [loclen],eax
    mov  [readblock],dword 1
    jmp  readhd
  chdir1:
    cmp  [esi],byte '.'     ; if '.'
    jnz  chdir2
    jmp  still
  chdir2:
   
    add  edi,location
    mov  ecx,11
    cld
    rep  movsb
   
    mov  eax,[loclen]
    add  eax,12
    mov  [loclen],eax
    mov  [readblock],dword 1
   
  readhd:
   
    call read_directory
    mov  [butid],21
    call drawbuttons
   
    jmp  still
   
  nonewdir:
   
    jmp  still
   
   
   
   
;   *********************************************
;   *******  WINDOW DEFINITIONS AND DRAW ********
;   *********************************************
   
   
draw_window:
   
    mov  eax,12                    ; function 12:tell os about windowdraw
    mov  ebx,1                     ; 1, start of draw
    int  0x40
   
                                   ; DRAW WINDOW
    mov  eax,0                     ; function 0 : define and draw window
    mov  ebx,50*65536+480          ; [x start] *65536 + [x size]
    mov  ecx,50*65536+290          ; [y start] *65536 + [y size]
    mov  edx,0x03ffffff            ; color of work area RRGGBB
    mov  esi,0x808899ff            ; color of grab bar  RRGGBB,8->color g
    mov  edi,0x008899ff            ; color of frames    RRGGBB
    int  0x40
   
                                   ; WINDOW LABEL
    mov  eax,4                     ; function 4 : write text to window
    mov  ebx,8*65536+8             ; [x start] *65536 + [y start]
    mov  ecx,0x10ffffff            ; color of text RRGGBB
    mov  edx,labelt                ; pointer to text beginning
    mov  esi,labellen-labelt       ; text length
    int  0x40
   
    mov  eax,8
    mov  ebx,20*65536+50
    mov  ecx,37*65536+12
    mov  edx,2
    mov  esi,[b_color]
    int  0x40
  nextbut:
    add  ebx,51*65536
    inc  edx
    int  0x40
    cmp  edx,4
    jb   nextbut
    ; UP
    mov  ebx,359*65536+50           ; [x start] *65536 + [x size]
    mov  edx,13                     ; button id
    int  0x40
    ; DOWN
   
    mov  ebx,410*65536+50          ; [x start] *65536 + [x size]
    mov  edx,14                    ; button id
    int  0x40
    mov  ebx,25*65536+40          ; draw info text with function 4
    mov  ecx,0xffffff
    mov  edx,buttonstring
    mov  esi,[buttonstringlen]
    mov  eax,4
    int  0x40
   
    call drawbuttons
   
    mov  eax,12                    ; function 12:tell os about windowdraw
    mov  ebx,2                     ; 2, end of draw
    int  0x40
   
    ret
   
   
   
drawbuttons:
   
    pusha
   
    mov  ebx,150*65536+310
    mov  ecx,50*65536+12
    mov  edx,21                 ;ID   21  36
 newb:
    cmp  dl,[butid]
    je   drawcur
    jmp  nodrawcur
 drawcur:
    mov  esi,0x6677cc
    jmp  drawbut
 nodrawcur:
    mov  esi,0x112288
 drawbut:
    mov  eax,8
    int  0x40
    cmp  [drawid],1
    je   drawtxt
   
    pusha
    sub  ebx,130*65536
    mov  bx,120
    add  edx,20
    mov  esi,0x3344aa
    int  0x40
    popa
   
 drawtxt:
    pusha
    sub  edx,21
    mov  esi,edx
    imul edx,12
    cmp  edx,[loclen]
    jg   no_dir_text
    mov  eax,4
    sub  ebx,125*65536
    shr  ecx,16
    mov  bx,cx
    add  bx,3
    shl  esi,2+16
    add  ebx,esi
    mov  ecx,0xffffff
    add  edx,root
    cmp  edx,root
    jne  no_yellow
    mov  ecx,0xffff00
  no_yellow:
    mov  esi,11
    int  0x40
  no_dir_text:
    popa
   
    pusha
    sub  edx,21
    mov  eax,edx
    xor  edx,edx
    mov  ebx,62
    mul  ebx
    add  eax,fileinfo
    mov  ebx,155*65536
    shr  ecx,16
    mov  bx,cx
    add  ebx,5*65536+3
    mov  ecx,0xffffff
    cmp  [eax+21],word 'SM'
    jnz  noasm
    mov  ecx,0x88ffff
  noasm:
    cmp  [eax+7],word 'OL'
    jnz  nofolt
    mov  ecx,0xffff00
  nofolt:
    cmp  [eax+7],word 'EL'
    jnz  nodelt
    mov  ecx,0x99aaee
  nodelt:
    mov  edx,eax
    mov  esi,57
    mov  eax,4
    int  0x40
    popa
   
    add  ecx,(13*65536)
    inc  edx
    cmp  edx,37
    jnz  newb
   
    popa
   
    ret
   
   
   
fileinfoblock:
   
   dd 0x0       ; read
   dd 0x0       ; first block
   dd 0x1       ; number of blocks to read
   dd 0x20000   ; ret
   dd 0x10000   ; work
filedir:
   times 12*10 db 32
   
   
   
read_directory:
   
    mov  edi,0x20000
    mov  eax,0
    mov  ecx,512
    cld
    rep  stosb
   
    mov  esi,location
    mov  edi,filedir
    mov  ecx,12*8
    cld
    rep  movsb
   
    mov  eax,[loclen]
    mov  [filedir+eax],byte 0
   
    mov  eax,[readblock]
    dec  eax
    mov  [fileinfoblock+4],eax
    mov  eax,58
    mov  ebx,fileinfoblock
    int  0x40
   
    cmp  eax,0
    jne  hd_read_error
   
    mov  [dirlen],ebx
    mov  esi,0x20000
    mov  edi,data_area+1024
    mov  ecx,512
    cld
    rep  movsb
   
    mov  ebx,1024
   
    ; command succesful
   
    mov  esi,data_area+1024
    mov  edi,fileinfo+11
    mov  edx,16
   
  newlineb:
   
    pusha               ; clear
    mov  al,32
    mov  ecx,58
    sub  edi,11
    cld
    rep  stosb
    popa
   
    mov  cl,[esi]       ; end of entries ?
    cmp  cl,6
    jnz  noib0
   
    mov  [edi-5],dword 'EOE '
    add  esi,32
    add  edi,62
    jmp  inf
   
  noib0:
   
    mov  cl,[esi+0]
    cmp  cl,0xe5
    je   yesdelfil
   
    mov  cl,[esi+11]    ; long fat32 name ?
    cmp  cl,0xf
    jnz  noib1
   
    mov  [edi-5],dword 'F32 '
    add  esi,32
    add  edi,62
    jmp  inf
   
  noib1:
   
    mov  eax,'DAT '     ; data or .. ?
   
    mov  cl,[esi+0]     ; deleted file
    cmp  cl,0xe5
    je   yesdelfil
    cmp  cl,0x0
    je   yesdelfil
    jmp  nodelfil
   yesdelfil:
    mov  eax,'DEL '
    jmp  ffile
  nodelfil:
   
    mov  cl,[esi+11]    ; folder
    and  cl,0x10
    jz   ffile
    mov  eax,'FOL '
    mov  [edi-5],eax
    mov  [edi+45],byte '-'
    jmp  nosize
   
  ffile:
   
    mov  [edi-5],eax
   
    pusha               ; size
    mov  eax,[esi+28]
    mov  esi,edi
    add  esi,37
    mov  ebx,10
    mov  ecx,8
  newnum:
    xor  edx,edx
    div  ebx
    add  dl,48
    mov  [esi],dl
    cmp  eax,0
    jz   zernum
    dec  esi
    loop newnum
  zernum:
    popa
  nosize:
   
    pusha                    ; date
    mov  [edi+17],dword '.  .'

    movzx eax,word [esi+24]
    shr  eax,9         ; year
    add  eax,1980
    mov  ecx,4
  newdel1:
    dec  ecx
    xor  edx,edx
    mov  ebx,10
    div  ebx
    add  edx,48
    mov  [edi+ecx+21],dl
    test ecx,ecx
    jnz  newdel1

    movzx eax,word [esi+24]
    shr  eax,5
    and  eax,0x0f
    mov  ecx,2
  newdel2:
    dec  ecx
    xor  edx,edx
    mov  ebx,10
    div  ebx
    add  edx,48
    mov  [edi+ecx+18],dl
    test ecx,ecx
    jnz  newdel2

    movzx eax,word [esi+24]
    and  eax,0x1f
    mov  ecx,2
  newdel3:
    dec  ecx
    xor  edx,edx
    mov  ebx,10
    div  ebx
    add  edx,48
    mov  [edi+ecx+15],dl
    test ecx,ecx
    jnz  newdel3

    popa
   
   
    pusha                    ; number
    mov  ecx,17
    sub  ecx,edx
    mov  eax,[readblock]
    dec  eax
    shl  eax,4
    add  eax,ecx
    xor  edx,edx
    mov  ebx,10
    div  ebx
    add  dl,48
    mov  [edi-8],dl          ;0001
    xor  edx,edx
    div  ebx
    add  dl,48
    mov  [edi-9],dl          ;0010
    xor  edx,edx
    div  ebx
    add  al,48
    add  dl,48
    mov  [edi-10],dl         ;0100
    mov  [edi-11],al         ;1000
    mov  [edi-7],byte '.'
    popa
   
    mov  ecx,8          ; first 8
    cld
    rep  movsb
    mov  [edi],byte '.'
    inc  edi
    mov  ecx,3          ; last 3
    cld
    rep  movsb
   
    add  esi,(32-11)
    add  edi,(60-12+2)
   
  inf:
   
    dec  edx
    cmp  edx,0
    jnz  newlineb
   
    ret
   
   
hd_read_error:
   
    mov  [winpar],byte 8
    mov  [winpar+3],byte 12
    mov  [winstat],word 1
    cmp  eax,1
    jnz  la1
    mov  [winpar+1],word 82
  la1:
    cmp  eax,2
    jnz  la2
    mov  [winpar+1],word 164
  la2:
    cmp  eax,3
    jnz  la3
    mov  [winpar+1],word 205
  la3:
    cmp  eax,4
    jnz  la4
    mov  [winpar+1],word 123
  la4:
    cmp  eax,5
    jnz  la5
    mov  [winpar+1],word 246
    dec  dword [readblock]
    add  [loclen],dword 12
   
  la5:
    cmp  eax,6
    jnz  la6
    mov  [winpar+1],word 287
  la6:
   
    sub  [loclen],dword 12
    jmp  thread
   
  getparam:
    xor  eax,eax
    mov  al,[butid]
    sub  eax,21
    imul eax,32
    add  eax,data_area+1024
    mov  esi,eax
    mov  edi,param
    mov  ecx,11
    cld
    rep  movsb
    ret
   
  delete:
    call getparam
    cmp  [param],0xe5
    je   still
    cmp  [param],byte '.'
    je   still
    mov  edi,notdelete
  cont:
    cld
    mov  esi,param
    mov  ecx,8
    repe cmpsb
    je   still
    add  edi,ecx
    cmp  edi,notdelete+48
    jb   cont
   
    mov  [winpar],byte 0
    cmp  [location+1],dword 'RAMD'
    jne  hddelete
    mov  [winpar+1],dword 0
    jmp  thread
hddelete:
    cmp  [location+1],dword 'HARD'
    jne  still
    mov  [winpar+1],word 41
    mov  [winpar+3],byte 12
    mov  [winstat],1
thread:
    mov  eax,5
    mov  ebx,20
    int  0x40
    mov  eax,51
    mov  ebx,1
    mov  ecx,alert
    mov  edx,0xff000
    int  0x40
    cmp  ah,0xff
    je   still
 closewait:
    mov  eax,23
    mov  ebx,20
    int  0x40
    cmp  eax,1
    jne  closewait1
    call draw_window
 closewait1:
    bt   [winstat],1
    jnc  closewait
    btr  [winstat],1
    call read_directory
    call drawbuttons
    jmp  still
   
  alert:
    call aldraw_window
   
    alstill:
   
    mov  eax,10                 ; wait here for event
    int  0x40
   
    cmp  eax,1                  ; redraw request ?
    je   alred
    cmp  eax,2                  ; key in buffer ?
    je   alkey
    cmp  eax,3                  ; button in buffer ?
    je   albutton
   
    jmp  alstill
   
  alred:                          ; redraw
    call aldraw_window
    jmp  alstill
   
  alkey:                        ; key
    mov  eax,2                  ; just read it and ignore
    int  0x40
    bt   [winstat],0
    jnc  alkey1
    cmp  ah,13
    je   alclose
    jmp  alstill
  alkey1:
    cmp  ah,27
    je   alclose
    cmp  ah,121
    je   rdd
    cmp  ah,110
    je   alclose
    cmp  ah,89
    je   rdd
    cmp  ah,78
    je   alclose                                                               
    jmp  alstill
   
  albutton:                     ; button
    mov  eax,17                 ; get id
    int  0x40
   
    cmp  ah,2                    ; button id=2 ?
    je  alclose
    cmp  ah,1
    jne  alnoclose
  rdd:
    mov  ebx,param
    mov  eax,32
    int  0x40
   
 alclose:
    bts  [winstat],1
    mov  eax,-1
    int  0x40                 ; close this program
 alnoclose:
    jmp  alstill
   
   
aldraw_window:
   
   
    mov  eax,12                    ; function 12:tell os about windowdraw
    mov  ebx,1                     ; 1, start of draw
    int  0x40
   
                                   ; DRAW WINDOW
    mov  eax,0                     ; function 0 : define and draw window
    mov  ebx,100*65536+260         ; [x start] *65536 + [x size]
    mov  ecx,100*65536+120         ; [y start] *65536 + [y size]
    mov  edx,0x02ffffff            ; color of work area RRGGBB,8->color gl
    mov  esi,0x00d50011            ; color of grab bar  RRGGBB,8->color gl
    mov  edi,0x00d50011            ; color of frames    RRGGBB
    int  0x40
   
    mov  eax,8                     ; function 8 : define and draw button
    mov  ecx,70*65536+20           ; [y start] *65536 + [y size]
    mov  edx,2                     ; button id
    mov  esi,0x6688dd
    bt   [winstat],0
    jc   butok
    mov  ebx,150*65536+50
    int  0x40
    sub  ebx,6553600
    dec  edx
    int  0x40
    jmp  aldrawtext
  butok:
    mov ebx,100*65536+50
    int 0x40
  aldrawtext:
                                   ; WINDOW LABEL
    mov  eax,4                     ; function 4 : write text to window
    mov  ebx,8*65536+8             ; [x start] *65536 + [y start]
    mov  ecx,0x10ddeeff            ; color of text RRGGBB
    movzx  edx,byte [winpar]
    add  edx,allabelt
    mov  esi,8                     ; text length
    int  0x40
    mov  ebx,8*65536+35           ; draw info text with function 4
    mov  ecx,0x224466
    movzx  edx,word [winpar+1]
    add  edx,altext
    mov  esi,41
    int  0x40
    mov  ebx,95*65536+50
    movzx  edx,byte [winpar+3]
    add  edx,param
    mov  esi,11
    int  0x40
    mov  ebx,65*65536+76
    not  ecx
    mov  esi,18
    bt   [winstat],0
    jc   ok
    mov  edx,albuttext
    int  0x40
    jmp  closewin
  ok:
    mov  edx,albuttext+18
    int 0x40
   
  closewin:
    mov  eax,12                    ; function 12:tell os about windowdraw
    mov  ebx,2                     ; 2, end of draw
    int  0x40
   
    ret
   
   
; DATA AREA
winpar: db 0,0,0,0
winstat dw 0
notdelete:  db 'HARDDISKRAMDISK FIRST   SECOND  THIRD   FOURTH  '
altext:
    db '      DO YOU REALLY WANT TO DELETE       '      ;0
    db '    THE OS DON"T SUPPORT HDD DELETING    '      ;41
    db 'NO HD BASE AND/OR FAT32 PARTITION DEFINED'      ;82
    db '    INVALID PARTITION AND/OR HD BASE     '      ;123
    db '      THIS FS DOESNT SUPPORTS YET        '      ;205
    db '               UNKNOWN FS                '      ;246
    db '              END OF FILE                '      ;287
    db '             FILE NOT FOUND              '      ;328
albuttext: db 'Yes             No         Ok       '
allabelt:
     db   'DELETINGWARNING '
   
buttonstring: db 'Re-read View(F3) Del(F8)                                 '
              db  0x18,'(PgUp) ',0x19,'(PgDn)'
   
buttonstringlen dd 72
drawid    db   0
butid     db   21
dirlen    dd   0x1
b_color   dd   0x6677cc
editor    db   'TINYPAD    '
param     db   '           ',0,'           '
text      db   '/                       '
loclen    dd  0
readblock dd  1
 labelt    db   'SYSTEM TREE'
 labellen:
   
   
root      db   'ROOTDIR     '
   
location:
   
          db    '/           /           /           /           '
          db    '/           /           /           /           '
          db    '/           /           /           /           '
          db    '/           /           /           /           '
          db    '/           /           /           /           '
          db    '/           /           /           /           '
          db    '/           /           /           /           '
          db    '/           /           /           /           '
   
fileinfo:
   
    db   '  00.              .     10.20  01.01.00                      '
    db   '  00.              .     10.20  01.01.00                      '
    db   '  00.              .     10.20  01.01.00                      '
    db   '  00.              .     10.20  01.01.00                      '
    db   '  00.              .     10.20  01.01.00                      '
    db   '  00.              .     10.20  01.01.00                      '
    db   '  00.              .     10.20  01.01.00                      '
    db   '  00.              .     10.20  01.01.00                      '
    db   '  00.              .     10.20  01.01.00                      '
    db   '  00.              .     10.20  01.01.00                      '
    db   '  00.              .     10.20  01.01.00                      '
    db   '  00.              .     10.20  01.01.00                      '
    db   '  00.              .     10.20  01.01.00                      '
    db   '  00.              .     10.20  01.01.00                      '
    db   '  00.              .     10.20  01.01.00                      '
    db   '  00.              .     10.20  01.01.00                      '
   
   
data_area:
   
I_END:
   
   
   
   